UMaps = {}

UMaps.CONFIG = "UMaps.txt"
-----------------
--Config Format--
--[[
"Out"
{
	"maplist"	//list of maps (keys must be numeric from 1 to num maps) (leave off .bsp)
	{
		"1"		"gm_construct"
    	"2"		"map_2"
    }
    "timemode"		"g" //Can be "g" (game time) or "r" (real time)
    "time"
    {
    	//Exapmle game time (change on set delay)
    	"h"		"0"		//hours
    	"m"		"15"	//minutes
    	"s"		"30"	//seconds
    	//Example real time	(change at specific time) (HH:MM in 24 hour format)
    	"1"		"6:00"
    	"2"		"12:00"
    	"3"		"18:00"
    }
    "alertlength"	"60"	//how long to display the alert before changing map (seconds) (default 60)
}
]]--

--------------------
--Define Functions--
--------------------
function UMaps.loadConfig()
	if not file.Exists( UMaps.CONFIG ) then
		return nil, "Config file \"" .. UMaps.CONFIG .. "\" does not exist."
	end

	local t = util.KeyValuesToTable( file.Read( UMaps.CONFIG ) )

	local tmp = {}
	for k, v in pairs( t.maplist ) do
		if not file.Exists( "../maps/" .. v .. ".bsp" ) then
			return nil, "Map " .. v .. ".bsp doesn not exist."
		end
		if not tonumber(k) then return nil, "Invalid key for map " .. v .. "." end
		tmp[tonumber(k)] = v
	end
	UMaps.nextmap = tmp[1]
	UMaps.maplist = tmp

	if not type( t.time ) == "table" then return nil, "Invalid time table." end
	UMaps.time = t.time

	if t.timemode == "r" then
		local cur = { os.date( "*t" )[4], os.date( "*t" )[5] }
		local next = { 24, 60 }
		local times = {}
		local delay
		
		for _, v in pairs( t.time ) do
			local h = tonumber( string.sub( 1, string.find( v, ":" ) ) )
			if not h or h >= 24 then
				return nil, "Invalid hour in time " .. v .. "."
			end

			local m = tonumber( string.sub( string.find( v, ":" ) ) )
			if not m or m >= 60 then
				return nil, "Invalid minute in time " .. v .. "."
			end

			table.insert( times, { h, m} )
		end

		table.sort( times, function( a, b ) if a[1]<=b[1] then return true elseif a[1]==b[1]then return a[2]<b[2] end end )
		for _,v in ipairs( times ) do
			if v[1] >= cur[1] and v[2] >= cur[2] then
				if v[1] <= next[1] and v[2] <= next[2] then
					next = v
				end
			end
		end

		if next[1] == 24 and next[2] == 60 then
			next = { times[1][1], times[1][2] }
		end

		if next[1] < cur[1] then
			delay = ( 24 - cur[1] + next[1] ) * 3600
		else
			delay = ( next[1] - cur[1] ) * 3600
		end
		if next[2] < cur[2] then
			delay = delay + ( ( 60 - cur[2] + next[2] ) * 60 )
		else
			delay = delay + ( next[2] - cur[2] ) * 60
		end
	elseif t.timemode == "g" then
		delay = ( t.time.h or 0 ) * 3600
		delay = delay + ( t.time.m or 0 ) * 60
		delay = delay + ( t.time.s or 0 )
	else
		return nil, "Invalid timemode \"" .. t.timemode .. "\"."
	end
	UMaps.timemode = t.timemode
	
	UMaps.alertlength = t.alertlength or 60

	return delay
end

function UMaps.saveConfig()
	local t = {}
	t.maplist = UMaps.maplist
	table.insert( t.maplist, 1, t.maplist[ table.getn ( t.maplist ) ] )
	table.remove( t.maplist, table.getn( t.maplist ) )
	t.timemode = UMaps.timemode
	t.time = UMaps.time
	t.alertlength = UMaps.alertlength
	local kv = util.TableToKeyValues( t )
	print( kv )
	file.Write( UMaps.CONFIG, kv )
end

function UMaps.error( msg )
	Msg( "[LS Error: UMaps] " .. msg )
end

function UMaps.sendTime( rp, show )
	show = show or false

	local rp = RecipientFilter()
	rp:AddAllPlayers()

	umsg.Start( "UMaps_timer", rp )
		umsg.String( UMaps.nextmap )
		umsg.Long( UMaps.loadtime - CurTime() + 2)
		umsg.Bool( show )
	umsg.End()
end

local nextsend
local shown = false
local minshown = false
local halfminshown = false
function UMaps.think()
	local rp = RecipientFilter()
	rp:AddAllPlayers()

	if CurTime() >= UMaps.loadtime then
		game.ConsoleCommand( "changelevel " .. UMaps.nextmap .. "\n" )
	else
		if math.floor( CurTime() ) >= math.floor( UMaps.loadtime - 60 ) and not minshown then
			minshown = true
			for _, v in pairs( player.GetAll() ) do
				v:ChatPrint( "One minute untill map change.")
			end
		elseif math.floor( CurTime() ) == math.floor( UMaps.loadtime - 30 ) and not halfminshown then
			halfminshown = true
			for _, v in pairs( player.GetAll() ) do
				v:ChatPrint( "30 seconds untill map change.")
			end
		end

		if not nextsend or ( CurTime() >= nextsend ) then
			if ( ( CurTime() >= UMaps.loadtime - UMaps.alertlength ) and not shown ) then
				UMaps.sendTime( rp, true )
				shown = true
			else
				UMaps.sendTime( rp, false )
			end
			nextsend = CurTime() + 2
		end
	end
end

function UMaps.playerSay( ply, str_txt, ts )
	if str_txt == "timeleft" then ply:ConCommand( "umaps_timeleft" )
	elseif str_txt == "nextmap" then ply:ConCommand( "umaps_nextmap" )
	end
end

function UMaps.initialSpawn( ply )
	UMaps.sendTime( ply, shown )
end

------------------
--Run Everything--
------------------

UMaps.delay, err = UMaps.loadConfig()
if err then
	UMaps.error( err )
	return
end
UMaps.saveConfig()

UMaps.loadtime = CurTime() + UMaps.delay

hook.Add( "Think", "UMaps.think", UMaps.think )
hook.Add( "PlayerSay", "UMaps.playerSay", UMaps.playerSay )
hook.Add( "PlayerInitialSpawn", "UMaps.initialSpawn", UMaps.initialSpawn )